//*****************************************************************************
// Description     : Definition of functions for controlling device
// Target Device   : APU101
// Target Platform : Linux, C
// Date            : July 2017
// Version         : 1.0
//*****************************************************************************
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h>

#include "sitcp.h"
#include "device.h"
#include "local_define.h"


//#########################################################################
//  Name      : device_write_3step
//  Function  : issue command 3 times with value of 0, 1, 0 respectively
//  Arguments :
//      in pCom     : pointer to the structure data for referring sockets
//      in cmd_addr : command address
//  Return :
//       0 : success
//      -n : failed at nth command
//  Note : 
//#########################################################################
int device_write_3step(DEVICE_COM_INFO *pCom, unsigned int cmd_addr)
{
    int ret;

    // issue command(1st time value is 0)
    ret = sitcp_write_16bit(pCom, cmd_addr, 0);
    if (ret < 0) {
        return(-1);
    }

    // issue command(2nd time value is 1)
    ret = sitcp_write_16bit(pCom, cmd_addr, 1);
    if (ret < 0) {
        return(-2);
    }

    // issue command(3rd time value is 0)
    ret = sitcp_write_16bit(pCom, cmd_addr, 0);
    if (ret < 0) {
        return(-3);
    }

    return(0);
}


//#########################################################################
//  Name      : device_start_measurement
//  Function  : start measurement
//  Arguments :
//      in pCom : pointer to the structure data for referring sockets
//  Return :
//       0 : success
//      -1 : failed at stop command
//      -2 : failed at start command
//  Note : 
//#########################################################################
int device_start_measurement(DEVICE_COM_INFO *pCom)
{
    int ret;

	// issue stop command first, in case of in the middle of measurement
    ret = sitcp_write_16bit(pCom, DEVICE_COMMAND_AQS, DEVICE_PARAM_AQS_STOP);
    if (ret < 0) {
        return(-1);
    }

	// issue start command
    ret = sitcp_write_16bit(pCom, DEVICE_COMMAND_AQS, DEVICE_PARAM_AQS_START);
    if (ret < 0) {
        return(-2);
    }

    return(0);
}


//#########################################################################
//  Name      : device_read_status
//  Function  : read some status information from device
//  Arguments :
//      in  pCom      : pointer to the structure data for referring sockets
//      out pStat     : pointer to the measurement status
//      out pTime     : pointer to the elapsed time
//      out pInputCps : pointer to the input count rate
//      out pThrouCps : pointer to the throughput count rate
//  Return :
//       0 : success
//      -n : failed at nth command
//  Note : 
//#########################################################################
int device_read_status(DEVICE_COM_INFO *pCom,
                       unsigned short *pStat, unsigned qword *pTime,
                       unsigned int *pInputCps, unsigned int *pThrouCps)
{
    int ret;
    unsigned int now_time;  // elapsed time(msec)

    //
    // read each status
	// (read all data before display
	// in order to obtain values at the same possible point in time)
    //

    // measurement status: 2byte value(receive 16bit length data)
    *pStat = sitcp_read_16bit(pCom, DEVICE_COMMAND_AQS, &ret);
    if (ret != 0) { return (-1); }

    // input count rate: 4byte value(receive 32bit length data)
    *pInputCps = sitcp_read_32bit(pCom, DEVICE_STATUS_ICR, &ret);
    if (ret != 0) { return (-2); }

    // throughput count rate: 4byte value(receive 32bit length data)
    *pThrouCps = sitcp_read_32bit(pCom, DEVICE_STATUS_TCR, &ret); 
    if (ret != 0) { return (-3); }

    // elapsed time: 6byte value(receive 48bit length data)
    *pTime = sitcp_read_48bit(pCom, DEVICE_STATUS_RTM, &ret);
    if (ret != 0) { return (-4); }

    //
    // display
    //
    now_time  = (unsigned int)(device_count2msec(*pTime));    // convert count to msec
    printf("time(msec): %5d/%5d.", now_time, (unsigned int)(MEASUREMENT_TIME));
    printf("   input rate(cps): %5d", *pInputCps);
    printf("   throughput rate(cps): %5d\n", *pThrouCps);

    return(0);
}


//#########################################################################
//  Name      : device_read_histogram
//  Function  : read histogram data from device
//  Arguments :
//      in  pCom    : pointer to the structure data for referring sockets
//      out rbuffer : pointer to the data buffer
//      in  nbyte   : request data size(byte)
//      in  timeout : receive timeout value(msec)
//  Return :
//       0 : received data size
//      -1 : failed to data request
//      -2 : failed to data receive
//  Note : This function converts histogram data from network byte order
//         to host byte order
//#########################################################################
int device_read_histogram(DEVICE_COM_INFO *pCom,
                          char *rbuffer, int nbyte, int timeout)
{
    int i, unit;
    int ret;
    int result;
    int wtimeout;
    int read_byte, recv_byte;
    unsigned int *p;

    result = 0;

    // request histogram data of CH1(=0)
    ret = sitcp_write_16bit(pCom, DEVICE_COMMAND_RQH, 0);

    if (ret >= 0) {  // succeeded to request histogram data
        recv_byte = 0;
        wtimeout  = timeout;

        while(1) {
            // update request data size
            read_byte = nbyte - recv_byte; 

            // receive histogram data via port for fast data transfer
            ret = sitcp_recv_data(pCom, &rbuffer[recv_byte], read_byte, wtimeout);

            if (ret <= 0) {
                // no data or receive error
                break;
            }

            // update received data size
            recv_byte += ret;

            if (recv_byte >= nbyte) {
                // finish when received data size reached to request one
                break;
            }

            wtimeout = 10;   // timeout value is 10ms after the second receiving
        }

        if (recv_byte > 0) { // succeeded to receive
            // set total received size
            result = recv_byte;

            // convert histogram data from network byte order to host byte order
            p = (unsigned int*)(rbuffer);
            unit = nbyte / sizeof(unsigned int);

            for(i = 0; i < unit; i++) {
                p[i] = ntohl(p[i]);
            }
        }
        else { // failed to receive
            result = -2;
        }
    }
    else { // failed to request
        result = -1;
    }

    return(result);
}
